import { createVNode as _createVNode, createElementVNode as _createElementVNode, createTextVNode as _createTextVNode, createElementBlock as _createElementBlock, createCommentVNode as _createCommentVNode, renderSlot as _renderSlot, Fragment, toDisplayString, normalizeProps, withCtx, renderList, withDirectives, vShow } from 'vue';
import { kebabToCamel } from './kebabCaseEscape';
import { McBaseTransition } from '../_transition_';
/**
 * 创建组件VNode
 * @param type 自定义组件
 * @param props 组件Props
 * @param children
 * @param patchFlag PatchFlags
 * @param dynamicProps
 * @returns VNode
 */
export function createComponentVNode(type, props, children, patchFlag, dynamicProps) {
    if (children && !Array.isArray(children)) {
        Object.entries(children).forEach(([key, child]) => {
            if (key !== '_' && child) {
                // ！！！强制使用 withCtx
                children[key] = withCtx(() => (Array.isArray(child()) ? child() : [child()]));
            }
        });
        if (!children._) {
            children._ = 1 /* SlotFlags.STABLE */;
        }
    }
    return _createVNode(type, normalizeProps(props), children, patchFlag, dynamicProps);
}
/**
 * 创建内置Dom元素VNode
 * @param type 元素类型
 * @param props 内置Props
 * @param children
 * @param patchFlag PatchFlags
 * @param dynamicProps
 * @returns VNode
 */
export function createElementVNode(type, props, children, patchFlag, dynamicProps) {
    return _createElementVNode(type, normalizeProps(props), Array.isArray(children) ? children : typeof children === 'string' ? toDisplayString(children) : [children], patchFlag, dynamicProps);
}
/**
 * 创建文本VNode
 */
export function createTextVNode(source, patch) {
    return patch ? _createTextVNode(toDisplayString(source), 1 /* PatchFlags.TEXT */) : _createTextVNode(toDisplayString(source));
}
/**
 * 创建Fragment
 * @param children Array
 * @param patchFlag 64 | 128 | 256
 * @returns Fragment VNode
 * @example
 * ```html
 * <div v-for="item in data">{{ item.id }}</div>
 * ```
 *
 * ```ts
 * createFragment(
 *     renderList(data, item => createElementVNode('div', null, item.id, PatchFlags.TEXT)),
 *     PatchFlags.UNKEYED_FRAGMENT
 * )
 * ```
 */
export function createFragment(children, patchFlag) {
    return _createElementBlock(Fragment, null, children, patchFlag);
}
/**
 * 创建基础过渡效果
 * @param name
 * @param effect
 * @param slots
 * @returns
 */
export function createTransition(name, effect, slots) {
    return createComponentVNode(McBaseTransition, {
        name,
        enterFrom: effect,
        leaveTo: effect
    }, slots);
}
export function renderSlot(slots, name, props, fallback, noSlotted) {
    const camelKey = kebabToCamel(name);
    return _renderSlot(slots, slots[camelKey] ? camelKey : name, props, fallback, noSlotted);
}
export function createDirectives(type, maybeSource, cb, keyed = false) {
    if (type === 'v-for' && cb) {
        return createFragment(renderList(maybeSource, cb), keyed ? 128 /* PatchFlags.KEYED_FRAGMENT */ : 256 /* PatchFlags.UNKEYED_FRAGMENT */);
    }
    else if (type === 'v-if') {
        const { condition, node, commentText = 'v-if' } = maybeSource;
        return !!condition ? (typeof node === 'function' ? node() : node) : _createCommentVNode(commentText, true);
    }
    else if (type === 'v-show') {
        const { condition, node } = maybeSource;
        return withDirectives(node, [[vShow, condition]]);
    }
}
