import { defineComponent, onMounted, toRefs, shallowReactive, ref, watch } from 'vue';
import { useThemeRegister, flattenTree, createElementVNode, createDirectives } from '../_utils_';
import { useElementBounding, throttledWatch, useTemplateRefsList } from '@vueuse/core';
import { anchorProps } from './interface';
import { mainCssr, lightCssr, darkCssr } from './styles';
export default defineComponent({
    name: 'Anchor',
    props: anchorProps,
    emits: ['change'],
    setup(props, { emit, expose }) {
        // theme register
        onMounted(() => {
            useThemeRegister({
                key: 'Anchor',
                main: mainCssr,
                light: lightCssr,
                dark: darkCssr
            });
        });
        const { options, bound, offsetTop, offsetBottom, type, showTrack, showMarker } = toRefs(props);
        const activeKey = ref();
        const indicatorBarOffsetTop = ref(0);
        const topList = shallowReactive({});
        const anchorElRef = ref();
        const anchorLinkElRefs = useTemplateRefsList();
        const { top: anchorTop } = useElementBounding(anchorElRef);
        const callChange = (target) => {
            emit('change', target);
        };
        const setActiveLink = (key) => {
            activeKey.value = key;
            const activeLinkEl = anchorLinkElRefs.value.find(link => link.hash === key);
            activeLinkEl && setIndicatorBarOffset(activeLinkEl);
        };
        const setIndicatorBarOffset = (target) => {
            const { top } = target.getBoundingClientRect();
            indicatorBarOffsetTop.value = top - anchorTop.value;
        };
        const createAnchorLink = (option) => {
            const { href, title, children } = option;
            return createElementVNode('div', {
                class: ['mc-anchor-link', activeKey.value === href ? 'mc-anchor-link--active' : '']
            }, [
                createElementVNode('a', {
                    ref: anchorLinkElRefs.value.set,
                    class: 'mc-anchor-link-title',
                    href,
                    onClick: e => {
                        e.preventDefault();
                        const { hash } = e.target;
                        if (hash === activeKey.value)
                            return;
                        window.location.hash = hash;
                        const target = document.getElementById(hash.slice(1));
                        target === null || target === void 0 ? void 0 : target.scrollIntoView();
                        callChange(hash);
                        setIndicatorBarOffset(e.target);
                    }
                }, [typeof title === 'string' ? title : title === null || title === void 0 ? void 0 : title()], 8 /* PatchFlags.PROPS */, ['href']),
                createDirectives('v-for', children !== null && children !== void 0 ? children : [], item => createAnchorLink(item))
            ], 2 /* PatchFlags.CLASS */);
        };
        const setTargetElement = (options) => {
            const flattenData = flattenTree(options, 'children');
            flattenData.forEach(item => {
                const id = item.href;
                const el = document.getElementById(id.slice(1));
                if (el) {
                    const { top } = useElementBounding(el);
                    topList[id] = top;
                }
            });
        };
        throttledWatch(topList, () => {
            // console.log(Object.entries(topList).map(([key, value]) => value.value));
            const activeLink = Object.entries(topList).find(([key, item]) => item.value <= bound.value + offsetTop.value && item.value >= bound.value - offsetBottom.value);
            setActiveLink((activeLink === null || activeLink === void 0 ? void 0 : activeLink[0]) || '');
        }, {
            throttle: 64
        });
        watch(options, () => {
            var _a;
            setTargetElement((_a = options.value) !== null && _a !== void 0 ? _a : []);
        }, {
            deep: true
        });
        onMounted(() => {
            var _a;
            setTargetElement((_a = options.value) !== null && _a !== void 0 ? _a : []);
        });
        expose({
            el: anchorElRef.value,
            scrollTo(href) {
                const target = document.getElementById(href.slice(1));
                target === null || target === void 0 ? void 0 : target.scrollIntoView({
                    behavior: 'smooth'
                });
                callChange(href);
            }
        });
        // main logic...
        return () => {
            var _a;
            return createElementVNode('div', { ref_key: 'anchorElRef', ref: anchorElRef, class: ['mc-anchor', `mc-anchor-type-${type.value}`] }, [
                createDirectives('v-for', (_a = options.value) !== null && _a !== void 0 ? _a : [], item => createAnchorLink(item)),
                type.value === 'bar'
                    ? createElementVNode('div', { class: 'mc-anchor-indicator' }, [
                        showTrack.value ? createElementVNode('div', { class: 'mc-anchor-indicator-track' }) : null,
                        showMarker.value ? createElementVNode('div', { class: 'mc-anchor-indicator-marker', style: { top: `${indicatorBarOffsetTop.value + 4}px` } }, null, 4 /* PatchFlags.STYLE */) : null
                    ])
                    : null
            ], 2 /* PatchFlags.CLASS */);
        };
    }
});
