"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDirectives = exports.renderSlot = exports.createTransition = exports.createFragment = exports.createTextVNode = exports.createElementVNode = exports.createComponentVNode = void 0;
const vue_1 = require("vue");
const kebabCaseEscape_1 = require("./kebabCaseEscape");
const _transition_1 = require("../_transition_");
/**
 * 创建组件VNode
 * @param type 自定义组件
 * @param props 组件Props
 * @param children
 * @param patchFlag PatchFlags
 * @param dynamicProps
 * @returns VNode
 */
function createComponentVNode(type, props, children, patchFlag, dynamicProps) {
    if (children && !Array.isArray(children)) {
        Object.entries(children).forEach(([key, child]) => {
            if (key !== '_' && child) {
                // ！！！强制使用 withCtx
                children[key] = (0, vue_1.withCtx)(() => (Array.isArray(child()) ? child() : [child()]));
            }
        });
        if (!children._) {
            children._ = 1 /* SlotFlags.STABLE */;
        }
    }
    return (0, vue_1.createVNode)(type, (0, vue_1.normalizeProps)(props), children, patchFlag, dynamicProps);
}
exports.createComponentVNode = createComponentVNode;
/**
 * 创建内置Dom元素VNode
 * @param type 元素类型
 * @param props 内置Props
 * @param children
 * @param patchFlag PatchFlags
 * @param dynamicProps
 * @returns VNode
 */
function createElementVNode(type, props, children, patchFlag, dynamicProps) {
    return (0, vue_1.createElementVNode)(type, (0, vue_1.normalizeProps)(props), Array.isArray(children) ? children : typeof children === 'string' ? (0, vue_1.toDisplayString)(children) : [children], patchFlag, dynamicProps);
}
exports.createElementVNode = createElementVNode;
/**
 * 创建文本VNode
 */
function createTextVNode(source, patch) {
    return patch ? (0, vue_1.createTextVNode)((0, vue_1.toDisplayString)(source), 1 /* PatchFlags.TEXT */) : (0, vue_1.createTextVNode)((0, vue_1.toDisplayString)(source));
}
exports.createTextVNode = createTextVNode;
/**
 * 创建Fragment
 * @param children Array
 * @param patchFlag 64 | 128 | 256
 * @returns Fragment VNode
 * @example
 * ```html
 * <div v-for="item in data">{{ item.id }}</div>
 * ```
 *
 * ```ts
 * createFragment(
 *     renderList(data, item => createElementVNode('div', null, item.id, PatchFlags.TEXT)),
 *     PatchFlags.UNKEYED_FRAGMENT
 * )
 * ```
 */
function createFragment(children, patchFlag) {
    return (0, vue_1.createElementBlock)(vue_1.Fragment, null, children, patchFlag);
}
exports.createFragment = createFragment;
/**
 * 创建基础过渡效果
 * @param name
 * @param effect
 * @param slots
 * @returns
 */
function createTransition(name, effect, slots) {
    return createComponentVNode(_transition_1.McBaseTransition, {
        name,
        enterFrom: effect,
        leaveTo: effect
    }, slots);
}
exports.createTransition = createTransition;
function renderSlot(slots, name, props, fallback, noSlotted) {
    const camelKey = (0, kebabCaseEscape_1.kebabToCamel)(name);
    return (0, vue_1.renderSlot)(slots, slots[camelKey] ? camelKey : name, props, fallback, noSlotted);
}
exports.renderSlot = renderSlot;
function createDirectives(type, maybeSource, cb, keyed = false) {
    if (type === 'v-for' && cb) {
        return createFragment((0, vue_1.renderList)(maybeSource, cb), keyed ? 128 /* PatchFlags.KEYED_FRAGMENT */ : 256 /* PatchFlags.UNKEYED_FRAGMENT */);
    }
    else if (type === 'v-if') {
        const { condition, node, commentText = 'v-if' } = maybeSource;
        return !!condition ? (typeof node === 'function' ? node() : node) : (0, vue_1.createCommentVNode)(commentText, true);
    }
    else if (type === 'v-show') {
        const { condition, node } = maybeSource;
        return (0, vue_1.withDirectives)(node, [[vue_1.vShow, condition]]);
    }
}
exports.createDirectives = createDirectives;
